#!/usr/bin/env python

## @file UART.py
# @author Andrew Furlong
# @date 1 MAY 2014
#
# This code performs serial comunication between the BBB Lighting Control and Monitoring Web Interface
# and the Microstick PIC32 series microcontroller that actually performs the switching and monitoring functions.
# Communication is achieved using UART protocol supported through the pySerial module.
#
# This script will be called from the command line by the php webapge.
# Command line parameters are of the form "-#A", where # is the plug
# number and A is a command.  Supported plug numbers are 1 and 2.
#
# Supported commands: \n
#       -#e enables a plug \n
#       -#d disables a plug \n
#       -#r read on/off state \n
#       -#c read power usage
#
# Commands are sent to attached PIC32s via UART.
# The results are printed to stdout and caught by php.

## import needed libraries and modules
import Adafruit_BBIO.UART as UART
import serial
import io
import sys
import getopt

## This function contains all of the code for this script.
def main(argv):
        
        # set up the UART interfaces
        UART.setup("UART1")
        UART.setup("UART2")

        # tell the program which hardwware ports to use and what speed to communicate at
        ser1 = serial.Serial(port = "/dev/ttyO1", baudrate = 9600)
        ser1.close()
        ser1.open()
        ser2 = serial.Serial(port = "/dev/ttyO2", baudrate = 9600)
        ser2.close()
        ser2.open()

        # confirm that the UART ports are open and ready to go
        if ser1.isOpen():
                print "Serial 1 is open!"
        if ser2.isOpen():
                print "Serial 2 is open!"

        # set up string to be printed when invalid flag or argument is passed, or when -h flag is passed
        helpString1 = "-h(print this help) -1(commands for serial 1) -2(command for serial 2) arguments: e(turn power on) d(turn power off) "
        helpString2 = "r(read switch state) c(read current level)"
        helpString = helpString1 + helpString2

        # check for valid flag and argument input
        try:
                opts, args = getopt.getopt(argv,"h1:2:")
        # handle exception for invalid arguements
        except getopt.GetoptError:
                print 'ERROR: type "test_UART.py -h" for help'
                sys.exit(2)
        # handle valid arguments
        for opt, arg in opts:
                if opt == '-h':             # handle help flag
                        print helpString
                        sys.exit()
                elif opt in ("-1"):         # handle device 1 flag
                        if arg == 'e':                      # handle enable argument
                                ser1.write("N")             # write character over UART to serial device 1
                                print 'ON command sent'     # print string to command line (will be passed to web page)
                                return 'ON command sent'    # return string in case string return is required by user software
                        elif arg == 'd':                    # handle disable argument
                                ser1.write("F")
                                print 'OFF command sent'
                                return 'OFF command  sent'
                        elif arg == 'r':                    # handle read state argument
                                ser1.write("S")
                                status = ser1.read(1)
                                print status
                                return status
                        elif arg == 'c':                    # handle read current argument
                                ser1.write("R")
                                current = ser1.read(5)
                                print current
                                return current
                elif opt in ("-2"):         # handle device 2 flag
                        if arg == 'e':
                                ser2.write("N")
                                print 'ON command sent'
                                return 'ON command sent'
                        elif arg == 'd':
                                ser2.write("F")
                                print 'OFF command sent'
                                return 'OFF command  sent'
                        elif arg == 'r':
                                ser2.write("S")
                                status = ser2.read(1)
                                print status
                                return status
                        elif arg == 'c':
                                ser2.write("R")
                                current = ser2.read(5)
                                print current
                                return current

if __name__ == "__main__":
   main(sys.argv[1:])
